//<![CDATA[
Number.prototype.pad = function(size) {
  var s = String(this);
  if(typeof(size) !== "number"){size = 2;}

  while (s.length < size) {s = "0" + s;}
  return s;
}

Date.prototype.stdTimezoneOffset = function() {
    var jan = new Date(this.getFullYear(), 0, 1);
    var jul = new Date(this.getFullYear(), 6, 1);
    return Math.max(jan.getTimezoneOffset(), jul.getTimezoneOffset());
}

Date.prototype.dst = function() {
    return this.getTimezoneOffset() < this.stdTimezoneOffset();
}

if (!Date.now) {
  Date.now = function() { return new Date().getTime(); };
}

var CLIMATE_STEPS = 10;
var form_submit_choice = "";
var g_controlMode = 0;

function addEvent(element, evnt, funct){
  if (element.attachEvent)
   return element.attachEvent('on'+evnt, funct);
  else
   return element.addEventListener(evnt, funct, false);
}

function addRow(tableID) {

  var table = document.getElementById(tableID);

  var rowCount = table.rows.length;
  var row = table.insertRow(rowCount);

  var cell = row.insertCell(0);
  cell.innerHTML = rowCount;

  cell = row.insertCell(1);
  var element = document.createElement("input");
  element.type = "text";
  element.name="climate_settings__temperature__index"+ (rowCount - 1);
  cell.appendChild(element);

  cell = row.insertCell(2);
  element = document.createElement("input");
  element.type = "text";
  element.name="climate_settings__time__index"+ (rowCount - 1);
  cell.appendChild(element);
}

function makeTempTimeTable(tableID) {
  for (var i = 0; i < CLIMATE_STEPS; ++i) {
    addRow(tableID);
  }
}

function discardZeroFields(tableID) {
  var elements;
  var timeIsInvalid;
  if (tableID == "TempTimeTable") {
    for (var i = 0; i < CLIMATE_STEPS; ++i) {
      /* zero temperature shall be valid only when time is set */
      timeIsInvalid = false;
      elements = document.getElementsByName("climate_settings__time__index"+ i);
      if (elements != null) {
        if (elements[0].value == "0") {
          elements[0].value = "";
          timeIsInvalid = true;
        }
      }
      if (timeIsInvalid) {
        elements = document.getElementsByName("climate_settings__temperature__index"+ i);
        if (elements != null) {
          if (elements[0].value == "0")
            elements[0].value = "";
        }
      }
    }
  }
}

function addOptions(select) {
  for (var i = 1; i <= 10; ++i) {
    var option = new Option (i, i);
    select.add(option);
  }
}

function onContactChange(target) {
  var hiddenContact = document.getElementById("ContactContact");
  var caller = target.target;
  if (caller == null)
    caller = target.srcElement;
  var key = caller.options[caller.selectedIndex].value;
  if (contacts.hasOwnProperty(key)) {
    hiddenContact.value = contacts[key];
  }
  // encore; set the field for the form as this one is no longer there
  hiddenContact = document.getElementById("hidden_contact_name");
  if (hiddenContact != null) {
    hiddenContact.value = key;
  }
  hiddenContact = document.getElementById("hidden_contact_contact");
  if (hiddenContact != null) {
    hiddenContact.value = contacts[key];
  }
}

function onPresettingChange(target) {
  var caller = target.target;
  if (caller == null)
    caller = target.srcElement;
  var key = caller.options[caller.selectedIndex].value;
  if (presettings.hasOwnProperty(key)) {
    // empty all fields first
    Preset.publicStaticClearFormValues("-", document.forms["climate_settings"], CLIMATE_STEPS);
    presettings[key].publicFillFormValues(key, document.forms["climate_settings"]);
  } else {
    SetSelectedText("Presettings", "hidden_preset");
  }
}

function optionizeOptions(elementID, sourceValuesArray, targetFunction) {
  var selectList = document.getElementById(elementID);
  if (selectList == null)
    return;
  //Create and append the options
  for (var key in sourceValuesArray) {
    if (sourceValuesArray.hasOwnProperty(key)) {
      var option = new Option (key, key);
      selectList.add(option);
    }
  }
  //selectList.onchange=OnCategoryChange(selectList);
  addEvent(
    selectList,
    'change',
    function(target) {targetFunction(target);}
  );
}

function packInfo(documentId) {
  if (form_submit_choice.length > 0) {
    var submit_info = document.getElementById(documentId);
    submit_info.value = form_submit_choice;
    return true;
  }
  return false;
}

function disableForm(form_name) {
  var form = document.forms[form_name];
  var elements = form.elements;
  for (var i = 0, len = elements.length; i < len; ++i) {
      //elements[i].readOnly = true;
      elements[i].disabled = true;
  }
}

function SetSelectedOption(id_source, id_destination) {
  var foundOption = false;

  var selected = document.getElementById(id_source);
  var selectList = document.getElementById(id_destination);
  if (selectList == null || selected == null)
    return;
  var source_text = selected.value;
  // try to select an existing option
  for (var key in selectList.options) {
    if (selectList.options.hasOwnProperty(key)) {
      if (selectList.options[key].text == source_text) {
        selectList.options[key].selected = true;
        foundOption = true;
        break;
      }
    }
  }
  if (!foundOption && (source_text != "")) {
    var option = new Option (source_text, source_text);
    selectList.add(option);
    selectList.selectedIndex = selectList.options.length - 1;
  }
}

function SetSelectedText(id_source, id_destination) {
  var source = document.getElementById(id_source);
  var target = document.getElementById(id_destination);
  if (source == null || target == null)
    return;
  target.value = source.value;
}

function currentInfoString(xmlnode) {
  var cycle = xmlnode.getElementsByTagName("cycle_values");
  if (cycle != null) {
    var running = false;
    var state = 0;
    var mode = 0;
    var step = 0;
    var round = 0;
    var duration = 0;
    var infoelement;
    var startingTime = null;
    var rounds = document.forms["climate_settings"].elements["climate_settings__cycle__rounds"];
    rounds = parseInt(rounds.options[rounds.selectedIndex].value);
    // state and mode
    var element = cycle[0].getElementsByTagName("started");
    if (element != null) {
      try {
        //if (parseInt(myTextContent(element[0])) > 0)
        {
          var today = new Date();
          var incomingValue = parseInt(myTextContent(element[0]));
          if (incomingValue > 0) {
            if (today.dst()) {
              incomingValue = incomingValue - (60 * 60);
            }

            startingTime = new Date(incomingValue * 1000);
            g_controlMode = 1;
          }
        }
      } catch (err) {
        startingTime = null;
      }
    }
    
    element = cycle[0].getElementsByTagName("active");
    if (element != null) {
      if (myTextContent(element[0]) == "1") {
        running = true;
      }
    }
    element = cycle[0].getElementsByTagName("state");
    if (element != null) {
      state = parseInt(myTextContent(element[0]));
    }
    element = cycle[0].getElementsByTagName("mode");
    if (element != null) {
      mode = parseInt(myTextContent(element[0]));
    }
    element = cycle[0].getElementsByTagName("step");
    if (element != null) {
      step = parseInt(myTextContent(element[0]));
    }
    element = cycle[0].getElementsByTagName("cycle");
    if (element != null) {
      round = parseInt(myTextContent(element[0]));
      
      if (round >= rounds) {
        g_controlMode = 2;
      }
    }
    element = cycle[0].getElementsByTagName("duration");
    if (element != null) {
      duration = parseInt(myTextContent(element[0]));
    }

    if (startingTime != null) {
      infoelement = document.getElementById("climate_start");
      if (infoelement != null) {
        infoelement.innerHTML = startingTime.getHours().pad(2) + ":" + startingTime.getMinutes().pad(2) + ":" + startingTime.getSeconds().pad(2) + " " + startingTime.getDate().pad(2) + "." + (startingTime.getMonth() + 1).pad(2) + "." + startingTime.getFullYear();
      }
      infoelement = document.getElementById("climate_end");
      if (infoelement != null) {
        var endTime;
        try {
          var currentTemp = parseInt(myTextContent(document.getElementById("climate_channel0")));
          var positivePerHour = parseInt(document.forms["climate_settings"].elements["climate_settings__gradient__positive"].value);
          var negativePerHour = parseInt(document.forms["climate_settings"].elements["climate_settings__gradient__negative"].value);

          var estimatedRunning = 0;
          var nodeTemp;
          var nodeTime;
          var firstStep = step;
          for (var j = round; j < rounds; j++) {
            for (var i = firstStep; i < CLIMATE_STEPS; ++i) {
              nodeTemp = parseInt(document.forms["climate_settings"].elements["climate_settings__temperature__index"+ (i)].value);
              nodeTime = parseInt(document.forms["climate_settings"].elements["climate_settings__time__index"+ (i)].value);
              
              if (!isNaN(nodeTemp) && (!isNaN(nodeTime) && nodeTime != 0)) {
                var dummy = Math.abs(nodeTemp - currentTemp);
                switch (mode) {
                  // when heating/cooling
                  default :
                    if (nodeTemp > currentTemp) {
                      estimatedRunning += ((dummy / positivePerHour) * 60 * 60 * 1000);
                    } else {
                      estimatedRunning += ((dummy / negativePerHour) * 60 * 60 * 1000);
                    }
                    estimatedRunning += (nodeTime * 60 * 1000);
                    break;
                  // in holding mode
                  case 3:
                    dummy = (nodeTime - duration)
                    if (dummy < 0)
                      dummy = 0;
                    estimatedRunning += (dummy * 60 * 1000);
                    break;
                }
                currentTemp = nodeTemp;
              }
            }
            firstStep = 0;
          }
          endTime = new Date();
          if (endTime.dst()) {
            estimatedRunning = estimatedRunning - (60 * 60 * 1000);
          }
          endTime = new Date(Date.now() + estimatedRunning);
          infoelement.innerHTML = endTime.getHours().pad(2) + ":" + endTime.getMinutes().pad(2) + ":" + endTime.getSeconds().pad(2) + " " + endTime.getDate().pad(2) + "." + (endTime.getMonth() + 1).pad(2) + "." + endTime.getFullYear();
        } catch (err) {
          infoelement.innerHTML = "???";
        }
      }
    }

    infoelement = document.getElementById("climate_status");
    infoelement = document.getElementById("climate_current_step");
    if (infoelement != null) {
      infoelement.innerHTML += (step + 1);
    }
    infoelement = document.getElementById("climate_current_round");
    if (infoelement != null) {
      infoelement.innerHTML = ((round < rounds) ? (round + 1) : round) + infoelement.innerHTML;
    }

    infoelement = document.getElementById("climate_detailmode");
    if (infoelement != null) {
      infoelement.style.filter = "none";
      switch (state) {
        default :
        case 0:
          infoelement.style.color = "white";
          infoelement.style.backgroundColor = "white";
          break;
        case 1:
          infoelement.style.color = "red";
          infoelement.style.backgroundColor = "red";
          break;
        case 2:
          infoelement.style.color = "blue";
          infoelement.style.backgroundColor = "blue";
          break;
      }
    }
    infoelement = document.getElementById("climate_mode");
    if (infoelement != null) {
      switch (mode) {
        default :
        case 0:
          infoelement.innerHTML = " ";
          break;
        case 1:
          infoelement.innerHTML = TEXT_HEATING;
          break;
        case 2:
          infoelement.innerHTML = TEXT_COOLING;
          break;
        case 3:
          infoelement.innerHTML = TEXT_HOLDING;
          infoelement = document.getElementById("climate_staying");
          if (infoelement != null) {
            infoelement.innerHTML = duration;
          }
          infoelement = document.getElementById("climate_stay");
          if (infoelement != null) {
            infoelement.innerHTML = document.forms["climate_settings"]["climate_settings__time__index" + step].value;
          }
          break;
      }
    }

    // description
    if (running) {
      // last status text
      element = cycle[0].getElementsByTagName("text");
      if (element != null) {
        infoelement = document.createElement("div");
        infoelement.innerHTML = "Letzte Meldung: "+ myTextContent(element[0]);
      }
    }

    infoelement = document.getElementById("climate_channel2");
    if (infoelement != null) {
      infoelement.innerHTML = document.forms["climate_settings"]["climate_settings__temperature__index" + step].value;
    }
    /*infoelement = document.getElementById("climate_channel2_unit");
    if (infoelement != null) {
      infoelement.innerHTML = "C";
    }*/
  }
}

function currentInfoData(xmlnode) {
  var stationID = "";
  var stationName = "";
  var timestamp = "";
  var channels, channel;
  var infoelement;
  var climate_counter = 0;
  var values = xmlnode.getElementsByTagName("avs");
  if (values == null)
    return false;
  values = values[0].getElementsByTagName("av");
  if (values == null)
    return false;

  for (var key in values) {
    if (values.hasOwnProperty(key) && (key != "length")) {
      stationID = values[key].getAttribute("si");
      stationName = values[key].getAttribute("sn");
      timestamp = values[key].getAttribute("ts");
      var timestamp_dummy = timestamp.split(" ");
      var timestamp_date = timestamp_dummy[0].split("-");
      var timestamp_time = timestamp_dummy[1].split(":");
      timestamp = new Date(timestamp_date[0], timestamp_date[1] - 1, timestamp_date[2], timestamp_time[0], timestamp_time[1], timestamp_time[2]);

      infoelement = document.getElementById("climate_number");
      if (infoelement != null) {
        infoelement.innerHTML = stationID;
      }
      infoelement = document.getElementById("climate_name");
      if ((infoelement != null) && (stationName.length > 0)) {
        infoelement.innerHTML = stationName;
      }
      if (timestamp != null) {
        infoelement = document.getElementById("climate_time");
        if (infoelement != null) {
          infoelement.innerHTML = timestamp.getHours().pad(2) + ":" + timestamp.getMinutes().pad(2) + ":" + timestamp.getSeconds().pad(2);
        }
        infoelement = document.getElementById("climate_date");
        if (infoelement != null) {
          infoelement.innerHTML = timestamp.getDate().pad(2) + "." + (timestamp.getMonth() + 1).pad(2) + "." + timestamp.getFullYear();
        }
      }

      channels = values[key].getElementsByTagName("c");
      if (channels != null) {
        channel = channels[0].getElementsByTagName("v");
        if (channel != null) {
          for (var measuring in channel) {
            if (channel.hasOwnProperty(measuring) && (measuring != "length")) {
              var cName = channel[measuring].getAttribute("cn");
              var cUnit = channel[measuring].getAttribute("cu");
              var cValue = channel[measuring].getAttribute("cv");
              
              infoelement = document.getElementById("climate_channel"+ climate_counter);
              infoelement.innerHTML = cValue;
              infoelement = document.getElementById("climate_channel"+ climate_counter +"_unit");
              infoelement.innerHTML = cUnit;

              climate_counter++;
              if (climate_counter > 2)
                break;
            }
          }
        }
      }
    }
  }
}

function fillCurrentInfo(server_address) {
  var xml = loadXML("GET", server_address +"/centipede?get=actual&_="+ Date.now());
  if (xml == null)
    return;

  xml = parseXML(xml);
  var actual = xml.getElementsByTagName("climate_control");
  if (actual != null) {
    currentInfoData(actual[0]);
    currentInfoString(actual[0]);
  }
}

function fillSvg(server_address, svg_element) {
  var childCount = 0;
  var xhr = getXmlHttpRequest();
  xhr.open("GET", server_address +"/output/output.svg?_="+ Date.now(), false);
  // Following line is just to be on the safe side;
  // not needed if your server delivers SVG with correct MIME type
  if (xhr.overrideMimeType) xhr.overrideMimeType("image/svg+xml");
  xhr.send("");
  if (xhr.status >= 300)
    throw "Status "+ xhr.status;
  if ('childElementCount' in svg_element) {
    childCount = svg_element.childElementCount;
  }
  else {
    if (svg_element.children) {
      childCount = svg_element.children.length;
    }
  }
  if (childCount > 0) {
    // remove old svg
    var children = svg_element.childNodes;
    for (var i = 0; i < children.length; ++i) {
      if (children[i].localName == "svg") {
        svg_element.removeChild(children[i]);
        break;
      }
    }
  }
  svg_element.appendChild(xhr.responseXML.documentElement);
}

function actualizeCurrentInfoData(server_address) {
  var active = document.forms["climate_settings"]["active"].value;
  var xml = loadXML("GET", server_address +"/cobra?"+ active +"&nohtml&_="+ Date.now());
  if (xml == null)
    return;

  xml = parseXML(xml);
  if (xml != null) {
    currentInfoData(xml);
  }
}

function climateOnLoad() {
  makeTempTimeTable("TempTimeTable");
  addOptions(document.getElementById("CycleRounds"));
  optionizeOptions("ContactName", contacts, onContactChange);
  optionizeOptions("Presettings", presettings, onPresettingChange);

  var parser = document.createElement('a');
  parser.href = window.location.href;
  baseUri = parser.protocol +"//"+ parser.hostname;

  try {
    fillClimateSettings(baseUri, "climate_settings");
  } catch (err) {
    //possibly no correct answer ...
  }
  try {
    fillCurrentInfo(baseUri);
  } catch (err) {
    //possibly no correct answer ...
    var infofield = document.getElementById("climate_current");
    if (infofield) {
      var infoelement = document.createElement("div");
      infoelement.innerHTML = "nicht erreichbar";
      infofield.appendChild(infoelement);
    }
  }

  var element;
  if (document.climate_settings.active.value == "1") {
    element = document.getElementById("start");
    if (element != null)
      element.style.display = "none";
    element = document.getElementById("update");
    if (element != null)
      element.style.display = "none";
    element = document.getElementById("restart");
    if (element != null)
      element.style.display = "none";

    // the status
    element = document.getElementsByClassName("information_stop");
    if (element != null)
      element[0].style.display = "none";
    element = document.getElementsByClassName("information_pause");
    if (element != null)
      element[0].style.display = "none";
    
    // disable fields when running; g_controlMode has to have a value now!
    if (g_controlMode > 0) {
      element = document.getElementById("Presettings");
      if (element != null)
        element.disabled = true;
      element = document.getElementById("ContactName");
      if (element != null)
        element.disabled = true;
    }
    disableForm("climate_settings");   
  } else {
    element = document.getElementById("stop");
    if (element != null)
      element.style.display = "none";
    element = document.getElementById("pause");
    if (element != null)
      element.style.display = "none";

    // the status
    element = document.getElementsByClassName("information_run");
    if (element != null)
      element[0].style.display = "none";
    switch (g_controlMode) {
      case 0 :
      case 2 :
        element = document.getElementsByClassName("information_pause");
        if (element != null)
          element[0].style.display = "none";
        element = document.getElementById("update");
        if (element != null)
          element.style.display = "none";
        break;
      case 1 :
        // disable fields when only pausing
        element = document.getElementById("Presettings");
        if (element != null)
          element.disabled = true;
        element = document.getElementById("ContactName");
        if (element != null)
          element.disabled = true;
      
        element = document.getElementsByClassName("information_stop");
        if (element != null)
          element[0].style.display = "none";
        break;
    }
  }
  element = document.getElementById("information_button");
  element.style.display = "inline";
  SetSelectedOption("hidden_preset", "Presettings");
  SetSelectedOption("hidden_contact_name", "ContactName");
  SetSelectedText("hidden_contact_contact", "ContactContact");
  discardZeroFields("TempTimeTable");

  var svgfield = document.getElementById("svgContainer");
  if (svgfield) {
    var infoelement = document.createElement("a");
    infoelement.href = baseUri +"/output/output.svg";
    infoelement.name = "Kurvengrafik";
    try {
      fillSvg(baseUri, svgfield);
      var spanelement = document.createElement("span");
      spanelement.className = "link-spanner";
      infoelement.appendChild(spanelement);
    } catch (err) {
      //possibly no correct answer ...
      infoelement.innerHTML = "Kurvengrafik";
    }
    svgfield.appendChild(infoelement);
  }
}

//]]>
